@@ -38,8 +38,8 @@
 
 extern XstTool *tool;
 
-GtkWidget *boot_table;
-GArray *boot_array;
+GtkWidget *boot_table = NULL;
+GArray *boot_array = NULL;
 
 const gchar *basic_boot_state = "\
 <ETableState> \
@@ -59,10 +59,6 @@ const gchar *adv_boot_state = "\
   </grouping> \
 </ETableState>";
 
-
-/* Static prototypes */
-void init_array (void);
-
 static int
 boot_col_count (ETableModel *etc, void *data)
 {
@@ -72,7 +68,11 @@ boot_col_count (ETableModel *etc, void *data)
 static int
 boot_row_count (ETableModel *etc, void *data)
 {
-	return boot_array->len;
+	if (boot_array)
+		return boot_array->len;
+
+	/* else */
+	return 0;
 }
 
 static void *
@@ -94,7 +94,7 @@ boot_value_at (ETableModel *etc, int col, int row, void *data)
 		return boot_value_image (node, FALSE);
 		break;
 	case COL_DEV:
-		return boot_value_image (node, FALSE);
+		return boot_value_device (node, FALSE);
 		break;
 	default:
 		return NULL;
@@ -145,7 +145,7 @@ boot_value_to_string (ETableModel *etc, int col, const void *value, void *data)
 static void
 boot_cursor_change (ETable *table, gint row, gpointer user_data)
 {
-	actions_set_sensitive (TRUE);
+	callbacks_actions_set_sensitive (TRUE);
 }
 
 static ETableExtras *
@@ -216,23 +216,26 @@ table_connect_signals (ETable *table)
 			    (gpointer)table);
 }
 
-extern void
-create_table (xmlNodePtr root)
+GtkWidget *
+table_create (void)
 {
-	ETableModel *model;
+	ETableModel  *model;
 	ETableExtras *extras;
-	ETable *table;
-	GtkWidget *container;
-	gchar *spec, *state;
+	ETable       *table;
+	gchar        *spec;
 
-	init_array ();
-	extras = create_extras ();
+	if (boot_table)
+		return NULL;
 	
+	extras = create_extras ();
 	model = e_table_simple_new (boot_col_count,
 				    boot_row_count,
+				    NULL,
 				    boot_value_at,
 				    boot_set_value_at,
 				    boot_is_cell_editable,
+				    NULL,
+				    NULL,
 				    boot_duplicate_value,
 				    boot_free_value,
 				    boot_initialize_value,
@@ -248,46 +251,52 @@ create_table (xmlNodePtr root)
 		xst_conf_set_string (tool, "spec", spec);
 	}
 
-	if (xst_dialog_get_complexity (tool->main_dialog) == XST_DIALOG_ADVANCED)
-		state = xst_conf_get_string (tool, "state_adv");
-	else
-		state = xst_conf_get_string (tool, "state_basic");
-	
-	if (!state) {
-		state = g_strdup (basic_boot_state);
-		xst_conf_set_string (tool, "state_basic", basic_boot_state);
-		xst_conf_set_string (tool, "state_adv", adv_boot_state);
-	}
-
-	boot_table = e_table_scrolled_new (E_TABLE_MODEL (model), extras, spec, state);
-	
+	boot_table = e_table_scrolled_new (E_TABLE_MODEL (model), extras, spec, basic_boot_state);
 	g_free (spec);
-	g_free (state);
+
+	g_return_val_if_fail (boot_table != NULL, NULL);
 
 	table = e_table_scrolled_get_table (E_TABLE_SCROLLED (boot_table));
 	table_connect_signals (table);
 	gtk_signal_connect (GTK_OBJECT (table), "cursor_change", boot_cursor_change, NULL);
-	
-	container = xst_dialog_get_widget (tool->main_dialog, "table_holder");
-	gtk_container_add (GTK_CONTAINER (container), boot_table);
-	gtk_widget_show (boot_table);
+
+	return boot_table;
 }
 
 void
-init_array (void)
+table_populate (xmlNodePtr root)
 {
-	xmlNodePtr node;
-	gint row;
+	xmlNodePtr  node;
+	gint        row;
+	ETable     *table;
+
+	g_return_if_fail (root != NULL);
+	
+	table = e_table_scrolled_get_table (E_TABLE_SCROLLED (boot_table));
 
 	boot_array = g_array_new (FALSE, FALSE, sizeof (xmlNodePtr));
-	node = xst_xml_doc_get_root (tool->config);
 	
-	for (node = xst_xml_element_find_first (node, "entry"), row = 0;
-		node;
-		node = xst_xml_element_find_next (node, "entry"), row++)
+	for (node = xst_xml_element_find_first (root, "entry"), row = 0;
+	     node != NULL;
+	     node = xst_xml_element_find_next (node, "entry"), row++)
 		
 		g_array_prepend_val (boot_array, node);
 
+	e_table_model_changed (table->model);
+}
+
+static gboolean
+boot_is_linux (xmlNodePtr node)
+{
+	xmlNodePtr n;
+	
+	g_return_val_if_fail (node != NULL, FALSE);
+
+	n = xst_xml_element_find_first (node, "image");
+	if (n)
+		return TRUE;
+	else
+		return FALSE;
 }
 
 void *
@@ -323,7 +332,7 @@ boot_value_type (xmlNodePtr node)
 	
 	g_return_val_if_fail (node != NULL, type);
 
-	n = xst_xml_element_find_first (node, "XstPartitionType");
+	n = xst_xml_element_find_first (node, "XstImageType");
 	if (n) {
 		gchar *buf;
 		
@@ -362,25 +371,50 @@ boot_value_type_char (xmlNodePtr node, gboolean bare)
 }
 
 void *
-boot_value_image (xmlNodePtr node, gboolean bare)
+boot_value_device (xmlNodePtr node, gboolean bare)
 {
-	gchar *buf;
-	
+	gchar *buf, *device;
+
 	g_return_val_if_fail (node != NULL, NULL);
 	
-	buf = xst_xml_get_child_content (node, "image");
-	if (!buf) {
+	if (boot_is_linux (node))
+		buf = xst_xml_get_child_content (node, "root");
+	else
 		buf = xst_xml_get_child_content (node, "other");
-		if (!buf)
-			return g_strdup ("");
+
+	if (buf == NULL)
+		buf = g_strdup ("");
+
+	if (bare)
+		device = buf;
+	else {
+		device = g_strdup_printf ("  %s", buf);
+		g_free (buf);
 	}
 
-	/* TODO: fix memory leak */
+	return device;
+}			
+
+void *
+boot_value_image (xmlNodePtr node, gboolean bare)
+{
+	gchar *buf, *image;
 	
+	g_return_val_if_fail (node != NULL, NULL);
+
+	if (boot_is_linux (node))
+		buf = xst_xml_get_child_content (node, "image");
+	else
+		buf = g_strdup ("");
+
 	if (bare)
-		return buf;
-	
-	return g_strdup_printf ("  %s", buf);
+		image = buf;
+	else {
+		image = g_strdup_printf ("  %s", buf);
+		g_free (buf);
+	}
+
+	return image;
 }
 
 void *
@@ -461,16 +495,22 @@ boot_value_set_label (xmlNodePtr node, const gchar *val)
 }
 
 void
-boot_value_set_image (xmlNodePtr node, const gchar *val)
+boot_value_set_image (xmlNodePtr node, const gchar *val, XstBootImageType type)
 {
 	xmlNodePtr n0;
+	gchar buf[6]; /* (strlen ('image') || strlen ('other')) + 1; */
 	
 	g_return_if_fail (node != NULL);
 
-	n0 = xst_xml_element_find_first (node, "image");
+	if (type == TYPE_LINUX)
+		strncpy (buf, "image", 6);
+	else
+		strncpy (buf, "other", 6);
+
+	n0 = xst_xml_element_find_first (node, buf);
 	if (val && strlen (val) > 0) {
 		if (!n0)
-			n0 = xst_xml_element_add (node, "image");
+			n0 = xst_xml_element_add (node, buf);
 		xst_xml_element_set_content (n0, val);
 	} else {
 		if (n0)
@@ -478,15 +518,6 @@ boot_value_set_image (xmlNodePtr node, const gchar *val)
 	}
 }
 
-void
-boot_value_set_dev (xmlNodePtr node, gchar *val)
-{
-	/* TODO: remove, use boot_value_set_image */
-	g_return_if_fail (node != NULL);
-
-	xst_xml_set_child_content (node, "other", val);
-}
-
 void
 boot_value_set_root (xmlNodePtr node, const gchar *val)
 {
@@ -532,9 +563,9 @@ boot_value_set_type (xmlNodePtr node, XstBootImageType type)
 	g_return_if_fail (node != NULL);
 
 	buf = type_to_label (type);
-	n0 = xst_xml_element_find_first (node, "XstPartitionType");
+	n0 = xst_xml_element_find_first (node, "XstImageType");
 	if (!n0)
-		n0 = xst_xml_element_add (node, "XstPartitionType");
+		n0 = xst_xml_element_add (node, "XstImageType");
 
 	xst_xml_element_set_content (n0, buf);
 	g_free (buf);
@@ -554,20 +585,33 @@ get_selected_node (void)
 	return NULL;
 }
 
-extern void
+void
 boot_table_update_state (void)
 {
 	ETable *table;
 	XstDialogComplexity complexity;
 	gchar *state;
 
+	g_return_if_fail (boot_table != NULL);
+
 	table = e_table_scrolled_get_table (E_TABLE_SCROLLED (boot_table));
+	g_return_if_fail (table != NULL);
+	
 	complexity = tool->main_dialog->complexity;
 
-	if (complexity == XST_DIALOG_BASIC)
+	if (complexity == XST_DIALOG_BASIC) {
 		state = xst_conf_get_string (tool, "state_basic");
-	else
+		if (state == NULL) {
+			state = g_strdup (basic_boot_state);
+			xst_conf_set_string (tool, "state_basic", state);
+		}
+	} else {
 		state = xst_conf_get_string (tool, "state_adv");
+		if (state == NULL) {
+			state = g_strdup (adv_boot_state);
+			xst_conf_set_string (tool, "state_adv", state);
+		}
+	}
 
 	e_table_set_state (table, state);
 	table_connect_signals (table);
