@@ -30,14 +30,24 @@
 #include "xst-ui.h"
 #include "xst-su.h"
 
+#include <bonobo.h>
+#include <config-archiver/archiver-client.h>
+
 #include <gnome.h>
 #include <parser.h>
 #include <memory.h>
 
 #include <stdio.h>
 #include <fcntl.h>
+#include <errno.h>
+
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <signal.h>
 
 #define XST_DEBUG 1
+/* Define this if you want the frontend to run the backend under strace */
+/*#define XST_DEBUG_STRACE_BACKEND*/
 
 enum {
 	FILL_GUI,
@@ -48,30 +58,35 @@ enum {
 static enum {
 	ROOT_ACCESS_NONE,
 	ROOT_ACCESS_SIMULATED,
+	ROOT_ACCESS_SIMULATED_DISABLED,
 	ROOT_ACCESS_REAL
 } root_access = ROOT_ACCESS_NONE;
 
 static GtkObjectClass *parent_class;
 static gint xsttool_signals [LAST_SIGNAL] = { 0 };
+static const gchar *XST_TOOL_EOR = "\n<!-- XST: end of request -->\n";
 
-static gboolean platform_unsupported_cb   (XstTool *tool, XstReportLine *rline);
-static gboolean platform_add_supported_cb (XstTool *tool, XstReportLine *rline);
-static gboolean platform_set_current_cb   (XstTool *tool, XstReportLine *rline);
-static gboolean report_finished_cb        (XstTool *tool, XstReportLine *rline);
+static gboolean platform_unsupported_cb   (XstTool *tool, XstReportLine *rline, gpointer data);
+static gboolean platform_add_supported_cb (XstTool *tool, XstReportLine *rline, gpointer data);
+static gboolean platform_set_current_cb   (XstTool *tool, XstReportLine *rline, gpointer data);
+static gboolean report_finished_cb        (XstTool *tool, XstReportLine *rline, gpointer data);
 
 static void report_dispatch (XstTool *tool);
 
 static gint xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b);
 
 static XstReportHookEntry common_report_hooks[] = {
-	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE  },
-	{ "platform_list",    platform_add_supported_cb,  XST_REPORT_HOOK_LOAD,     TRUE  }, 
-	{ "platform_unsup",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE },
-	{ "platform_undet",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE },
-	{ "platform_success", platform_set_current_cb,    XST_REPORT_HOOK_LOAD,     FALSE }, 
-	{ 0,                  NULL,                       -1,                       FALSE }
+/*        key                 function                    type                      allow_repeat user_data */
+	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE,        NULL },
+	{ "platform_list",    platform_add_supported_cb,  XST_REPORT_HOOK_LOAD,     TRUE,        NULL }, 
+	{ "platform_unsup",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE,       NULL },
+	{ "platform_undet",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE,       NULL },
+	{ "platform_success", platform_set_current_cb,    XST_REPORT_HOOK_SAVE,     FALSE,       NULL }, 
+	{ NULL,               NULL,                       -1,                       FALSE,       NULL }
 };
 
+static gchar *location_id;    /* Location in which we are editing */
+
 /* --- Report hook callbacks --- */
 
 static void
@@ -99,17 +114,16 @@ platform_list_unselect_row_cb (GtkCList *clist, gint row, gint column, GdkEvent
 static void
 platform_dialog_close_cb (GtkWidget *dialog, XstTool *tool)
 {
-	exit (1);
+	exit (0);
 }
 
 static void
 platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 {
 	if (ret == -1 || ret == 1)
-		exit (1);
+		exit (0);
 
-	/* Locate the selected platform and set it up as the current one. Prepare to
-	 * invoke the backend again, this time with the current platform specified. */
+	/* Locate the selected platform and set it up as the current one. */
 
 	if (tool->current_platform)
 		xst_platform_free (tool->current_platform);
@@ -121,61 +135,21 @@ platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 
 	g_assert (tool->current_platform);
 
-	tool->report_dispatch_pending = FALSE;
-	tool->report_finished = TRUE;
-	tool->run_again = TRUE;
-
-	report_dispatch (tool);
-
 	gtk_signal_disconnect_by_func (GTK_OBJECT (tool->platform_dialog),
 				       platform_dialog_close_cb,
 				       tool);
-	
 }
 
 static gboolean
-platform_unsupported_cb (XstTool *tool, XstReportLine *rline)
+platform_unsupported_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
-	GSList *list;
-	XstPlatform *platform;
-	char *platform_text[1];
-
-	tool->report_dispatch_pending = TRUE;
-
-	/* Fill in the platform GtkCList */
-
-	gtk_clist_clear (GTK_CLIST (tool->platform_list));
-
-	for (list = tool->supported_platforms_list; list;
-	     list = g_slist_next (list))
-	{
-		platform = (XstPlatform *) list->data;
-
-		platform_text [0] = (char *) xst_platform_get_name (platform);
-		gtk_clist_append (GTK_CLIST (tool->platform_list), platform_text);
-	}
-
-	/* Prepare dialog and show it */
-
-	tool->platform_selected_row = -1;
-
-	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "select-row",
-			    platform_list_select_row_cb, tool);
-	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "unselect-row",
-			    platform_list_unselect_row_cb, tool);
-	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "clicked",
-			    platform_unsupported_clicked_cb, tool);
-	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "close",
-			    platform_dialog_close_cb, tool);
-
-	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
-	gtk_widget_show (tool->platform_dialog);
+	tool->run_again = TRUE;
 
 	return TRUE;
 }
 
 static gboolean
-platform_add_supported_cb (XstTool *tool, XstReportLine *rline)
+platform_add_supported_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
 	XstPlatform *platform;
 
@@ -187,7 +161,7 @@ platform_add_supported_cb (XstTool *tool, XstReportLine *rline)
 }
 
 static gboolean
-platform_set_current_cb (XstTool *tool, XstReportLine *rline)
+platform_set_current_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
 	XstPlatform *platform;
 
@@ -198,15 +172,21 @@ platform_set_current_cb (XstTool *tool, XstReportLine *rline)
 		xst_platform_free (tool->current_platform);
 
 	tool->current_platform = platform;
+	tool->run_again = FALSE;
 	return TRUE;
 }
 
 /* --- Other Report Hooks --- */
 
 static gboolean
-report_finished_cb (XstTool *tool, XstReportLine *rline)
+report_finished_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
+	tool->report_dispatch_pending = FALSE;
 	tool->report_finished = TRUE;
+
+	gtk_widget_hide (tool->report_window);
+	gtk_main_quit ();
+	
 	return TRUE;
 }
 
@@ -404,9 +384,6 @@ report_dispatch (XstTool *tool)
 		gdk_input_remove (tool->input_id);
 		tool->input_id = 0;
 	}
-
-	if (tool->report_finished && !tool->report_dispatch_pending)
-		gtk_main_quit ();
 }
 
 static void
@@ -414,50 +391,64 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 {
 	XstTool *tool;
 	XstReportLine *rline;
-	char c;
+	gchar buffer [512];
+	gint n, i = 0;
 
 	tool = XST_TOOL (data);
+	if (tool->input_block)
+		return;
 
-#warning FIXME: make this not suck
-	if (!tool->line) {
-		tool->line = g_malloc(1024);
-		tool->line_len = 0;
-		tool->line [0] = '\0';
-	}
+	if (!tool->line)
+		tool->line = g_string_new ("");
 
-	/* NOTE: The read() being done here is inefficient, but we're not
-	 * going to handle any large amount of data */
+	while ((n = read (fd, buffer, sizeof (buffer) - 1)) != 0) {
 
-	if (read (fd, &c, 1) > 0) {
-		if (c == '\n') {
-			/* End of line */
-
-		        /* Report line; add to list */
-			rline = xst_report_line_new_from_string (tool->line);
+		buffer [n] = 0;
+		
+		for (i = 0; (i < n); i++) {
+			gchar c = buffer [i];
 			
-			if (rline)
-				tool->report_line_list = g_slist_append (tool->report_line_list,
-									 rline);
+			if (c == '\n') {
+				/* End of line */
+				
+				/* Report line; add to list */
+				rline = xst_report_line_new_from_string (tool->line->str);
+				
+				if (rline)
+					tool->report_line_list = g_slist_append (
+						tool->report_line_list, rline);
 
-			tool->line [0] = '\0';
-			tool->line_len = 0;
-		} else {
-			/* Add character to end of current line */
+				g_string_assign (tool->line, "");
 
-			tool->line = g_realloc (tool->line, MAX (tool->line_len + 2, 1024));
-			tool->line [tool->line_len] = c;
-			tool->line_len++;
-			tool->line [tool->line_len] = '\0';
+				if (!tool->report_dispatch_pending) {
+					report_dispatch (tool);
+					if (tool->report_finished) {
+						i++;
+						goto full_break;
+					}
+				}
+			} else {
+				/* Add character to end of current line */
+				g_string_append_c (tool->line, buffer [i]);
+			}
 		}
 	}
-	else
-	{
+
+ full_break:
+
+	if (n <= 0) {
 		/* Zero-length read; pipe closed unexpectedly */
-		/* FIXME: handle this at the UI level correctly. */
 
 		tool->report_finished = TRUE;
 	}
 
+	if (tool->report_finished) {
+		if (n > 0)
+			tool->xml_document = g_string_new (&buffer [i]);
+		g_string_free (tool->line, TRUE);
+		tool->line = NULL;
+	}
+	
 	if (!tool->report_dispatch_pending)
 		report_dispatch (tool);
 	
@@ -468,13 +459,19 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 	}
 }
 
+static gboolean
+report_window_close_cb (GtkWidget *window, gpointer data)
+{
+	gtk_widget_hide (window);
+	return TRUE;
+}
+
 static void
-report_progress (XstTool *tool, int fd, const gchar *label)
+report_progress (XstTool *tool, const gchar *label)
 {
 #if 0
 	gint cb_id;
 #endif	
-
 	tool->timeout_done = FALSE;
 	tool->report_list_visible = FALSE;
 	tool->report_finished = FALSE;
@@ -483,16 +480,27 @@ report_progress (XstTool *tool, int fd, const gchar *label)
 	xst_tool_clear_supported_platforms (tool);
 	report_clear_lines (tool);
 
-	gtk_label_set_text (GTK_LABEL (tool->report_label), label);
 #if 0
 	set_arrow (tool, GTK_ARROW_DOWN);
 	gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress), 0.0);
 #endif
-	tool->input_id = gtk_input_add_full (fd, GDK_INPUT_READ, report_progress_tick,
-					     NULL, tool, NULL);
 
-	gtk_widget_show (tool->report_window);
-	
+	if (label) {
+		gtk_label_set_text (GTK_LABEL (tool->report_label), label);
+		gtk_signal_connect_after (GTK_OBJECT (tool->report_window), "delete-event",
+					  GTK_SIGNAL_FUNC (report_window_close_cb), NULL);
+		gtk_widget_show_all (tool->report_window);
+		while (gtk_events_pending ())
+		{
+			gtk_main_iteration ();
+			usleep (5000);
+		}
+	}
+
+	tool->input_id = gtk_input_add_full (tool->backend_read_fd, GDK_INPUT_READ,
+					     report_progress_tick, NULL, tool, NULL);
+	tool->input_block = FALSE;
+
 	gtk_main ();
 
 	if (tool->input_id)
@@ -516,29 +524,387 @@ report_progress (XstTool *tool, int fd, const gchar *label)
 #endif
 }
 
+static void
+xst_tool_run_platform_dialog (XstTool *tool)
+{
+	GSList *list;
+	XstPlatform *platform;
+	char *platform_text[1];
+
+	/* Fill in the platform GtkCList */
+
+	gtk_clist_clear (GTK_CLIST (tool->platform_list));
+
+	for (list = tool->supported_platforms_list; list;
+	     list = g_slist_next (list))
+	{
+		platform = (XstPlatform *) list->data;
+
+		platform_text [0] = (char *) xst_platform_get_name (platform);
+		gtk_clist_append (GTK_CLIST (tool->platform_list), platform_text);
+	}
+
+	/* Prepare dialog and show it */
+
+	tool->platform_selected_row = -1;
+
+	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "select-row",
+			    platform_list_select_row_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "unselect-row",
+			    platform_list_unselect_row_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "clicked",
+			    platform_unsupported_clicked_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "close",
+			    platform_dialog_close_cb, tool);
+
+	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
+
+	gnome_dialog_run (GNOME_DIALOG (tool->platform_dialog));
+}
+
+static void
+xst_tool_process_startup (XstTool *tool)
+{
+	/* let's process those startup reports. */
+	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
+	report_progress (tool, NULL);
+	if (tool->run_again) {
+		xst_tool_run_platform_dialog (tool);
+		g_assert (tool->current_platform);
+		
+		if (root_access == ROOT_ACCESS_SIMULATED)
+			root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+		
+		xst_tool_run_set_directive (tool, NULL, NULL, "platform_set",
+					    tool->current_platform->name, NULL);
+
+		if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+			root_access = ROOT_ACCESS_SIMULATED;
+	}
+}
+
+static void
+xst_tool_init_backend (XstTool *tool)
+{
+	int fd_read [2], fd_write [2];
+
+	g_return_if_fail (tool != NULL);
+	g_return_if_fail (XST_IS_TOOL (tool));
+	g_return_if_fail (tool->script_path != NULL);
+	g_return_if_fail (tool->backend_pid < 0);
+	
+	pipe (fd_read);
+	pipe (fd_write);
+
+	tool->backend_pid = fork ();
+	if (tool->backend_pid < 0) {
+		g_warning ("Unable to fork.");
+		tool->backend_pid = -1;
+		return;
+	} else if (tool->backend_pid) {
+		/* Parent */
+
+		close (fd_read  [1]); /* Close writing end of read pipe */
+		close (fd_write [0]); /* Close reading end of write pipe */
+
+		tool->backend_read_fd  = fd_read  [0];
+		tool->backend_write_fd = fd_write [1];
+
+		fcntl (tool->backend_read_fd, F_SETFL, 0);  /* Let's block */
+
+		xst_tool_process_startup (tool);
+	} else {
+		/* Child */
+
+		/* For the child, things are upside-down */
+		close (fd_read  [0]); /* Close reading end of read pipe */
+		close (fd_write [1]); /* Close writing end of write pipe */
+
+		dup2 (fd_read  [1], STDOUT_FILENO);
+		dup2 (fd_write [0],  STDIN_FILENO);
+
+		if (tool->current_platform)
+			execl (tool->script_path, tool->script_path,
+			       "--progress", "--report", "--platform",
+			       xst_platform_get_name (tool->current_platform), NULL);
+		else
+#ifndef XST_DEBUG_STRACE_BACKEND
+			execl (tool->script_path, tool->script_path, "--progress",
+			       "--report", NULL);
+#else		
+		execl ("/usr/bin/strace", "/usr/bin/strace", "-f", tool->script_path, "--progress",
+		       "--report", NULL);
+#endif					
+		
+		g_error ("Unable to run backend: %s", tool->script_path);
+	}
+}
+
+static void
+xst_tool_kill_backend (XstTool *tool, gpointer data)
+{
+	g_return_if_fail (tool != NULL);
+	g_return_if_fail (XST_IS_TOOL (tool));
+	
+	/* The backend was never called. No problem! */
+	/* For further reference check http://www.xs4all.nl/~pjbrink/apekool/050/045-en.html */
+	if (tool->backend_pid < 0)
+		return;
+
+	if (root_access == ROOT_ACCESS_SIMULATED)
+		root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+		
+	xst_tool_run_set_directive (tool, NULL, _("Closing tool."), "end", NULL);
+
+	if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+		root_access = ROOT_ACCESS_SIMULATED;
+	
+	waitpid (tool->backend_pid, NULL, 0);
+}
+
+static gboolean
+xst_tool_end_of_request (GString *string)
+{
+	gchar *str;
+	gint len, eorlen, i;
+
+	str = string->str;
+	len = strlen (str);
+	eorlen = strlen (XST_TOOL_EOR);
+	if (len < eorlen)
+		return FALSE;
+
+	for (i = 0; i < eorlen; i++)
+		if (XST_TOOL_EOR[eorlen - i - 1] != str[len - i - 1])
+			return FALSE;
+
+	g_string_truncate (string, len - eorlen + 1);
+
+	return TRUE;
+}
+
+static xmlDoc *
+xst_tool_read_xml_from_backend (XstTool *tool)
+{
+	char buffer [4096];
+	int t;
+	xmlDoc *xml;
+
+	if (!tool->xml_document)
+		return NULL;
+	
+	fcntl (tool->backend_read_fd, F_SETFL, O_NONBLOCK);
+
+	while (! xst_tool_end_of_request (tool->xml_document)) {
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+
+		t = read (tool->backend_read_fd, buffer, sizeof (buffer) - 1);
+
+		if (t == 0)
+			break;
+		if (t == -1)
+			t = 0;
+
+		buffer [t] = 0;
+		g_string_append (tool->xml_document, buffer);
+	}
+	
+	fcntl (tool->backend_read_fd, F_SETFL, 0);
+
+	if (tool->xml_document->str[0] == '<') {
+		xml = xmlParseDoc (tool->xml_document->str);
+	} else {
+		xml = NULL;
+	}
+
+	g_string_free (tool->xml_document, TRUE);
+	tool->xml_document = NULL;
+
+	return xml;
+}
+
+static void
+xst_tool_send_directive (XstTool *tool, const gchar *directive, va_list ap)
+{
+	GString *directive_line;
+	gchar *arg;
+	FILE *f;
+
+	directive_line = g_string_new (directive);
+	while ((arg = va_arg (ap, char *)) != NULL) {
+		g_string_append (directive_line, "::");
+		g_string_append (directive_line, arg);
+	}
+	va_end (ap);
+	g_string_append_c (directive_line, '\n');
+	
+	if (tool->backend_pid < 0)
+		xst_tool_init_backend (tool);
+
+	f = fdopen (dup (tool->backend_write_fd), "w");
+	fprintf (f, directive_line->str);
+	fclose (f);
+
+	g_string_free (directive_line, TRUE);
+}
+
+xmlDoc *
+xst_tool_run_get_directive (XstTool *tool, const gchar *report_sign, const gchar *directive, ...)
+{
+	va_list ap;
+	xmlDoc *xml;
+	const gchar *directive_ptr;
+	
+	g_return_val_if_fail (tool != NULL, NULL);
+	g_return_val_if_fail (XST_IS_TOOL (tool), NULL);
+
+	directive_ptr = directive;
+	va_start (ap, directive);
+	xst_tool_send_directive (tool, directive_ptr, ap);
+
+	/* FIXME: Instead of doing the following, we should pass around a value describing
+	 * tool I/O mode (as opposed to a string to report_progress ()). */
+	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
+
+	xmlSubstituteEntitiesDefault (TRUE);
+
+	/* LibXML support for parsing from memory is good, but parsing from
+	 * opened filehandles is not supported unless you write your own feed
+	 * mechanism. Let's just load it all into memory, then. Also, refusing
+	 * enormous documents can be considered a plus. </dystopic> */
+	
+	if (tool->xml_document == NULL)
+		tool->xml_document = g_string_new ("");
+	
+	if (location_id == NULL)
+		report_progress (tool, _(report_sign));
+
+	xml = xst_tool_read_xml_from_backend (tool);
+	
+	return xml;
+}
+
+xmlDoc *
+xst_tool_run_set_directive (XstTool *tool, xmlDoc *xml,
+			    const gchar *report_sign, const gchar *directive, ...)
+{
+	va_list ap;
+	FILE *f;
+	xmlDoc *xml_out;
+	const gchar *directive_ptr;
+	
+	/* don't actually run if we are just pretending */
+	if (root_access == ROOT_ACCESS_SIMULATED) {
+		g_warning (_("Skipping set directive..."));
+		return NULL;
+	}
+
+	g_return_val_if_fail (tool != NULL, NULL);
+	g_return_val_if_fail (XST_IS_TOOL (tool), NULL);
+
+	directive_ptr = directive;
+	va_start (ap, directive);
+	xst_tool_send_directive (tool, directive_ptr, ap);
+
+	if (xml) {
+		f = fdopen (dup (tool->backend_write_fd), "w");
+		xmlDocDump (f, xml);
+		fprintf (f, XST_TOOL_EOR);
+		fclose (f);
+	}
+
+	/* FIXME: Instead of doing the following, we should pass around a value describing
+	 * tool I/O mode (as opposed to a string to report_progress ()). */
+	tool->report_hook_type = XST_REPORT_HOOK_SAVE;
+
+	if (location_id == NULL)
+		report_progress (tool, report_sign? _(report_sign): NULL);
+
+	/* This is tipicaly to just read the end of request string,
+	   but a set directive may return some XML too. */
+	xml_out = xst_tool_read_xml_from_backend (tool);
+	return xml_out;
+}
+
+
+gboolean
+xst_tool_load (XstTool *tool)
+{
+	g_return_val_if_fail (tool != NULL, FALSE);
+	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);	
+	g_return_val_if_fail (tool->script_path, FALSE);
+  
+	if (tool->config) {
+		xmlFreeDoc (tool->config);
+		tool->config = NULL;
+	}
+
+	if (tool->run_again)
+		return TRUE;
+
+	if (location_id == NULL) {
+		tool->config = xst_tool_run_get_directive
+			(tool, _("Scanning your system configuration."), "get", NULL);
+	} else {
+		CORBA_Environment ev;
+		ConfigArchiver_Archive archive;
+		ConfigArchiver_Location location;
+		gchar *backend_id;
+
+		CORBA_exception_init (&ev);
+
+		archive = bonobo_get_object ("archive:global-archive", "IDL:ConfigArchiver/Archive:1.0", &ev);
+
+		if (BONOBO_EX (&ev) || archive == CORBA_OBJECT_NIL) {
+			g_critical ("Could not resolve the archive moniker");
+			return FALSE;
+		}
+
+		location = ConfigArchiver_Archive_getLocation (archive, location_id, &ev);
+
+		if (BONOBO_EX (&ev) || location == CORBA_OBJECT_NIL) {
+			g_critical ("Could not get the location %s", location_id);
+			return FALSE;
+		}
+
+		backend_id = strrchr (tool->script_path, '/');
+
+		if (backend_id != NULL)
+			backend_id++;
+		else
+			backend_id = tool->script_path;
+
+		tool->config = location_client_load_rollback_data (location, NULL, 0, backend_id, TRUE, &ev);
+
+		bonobo_object_release_unref (location, NULL);
+		bonobo_object_release_unref (archive, NULL);
+
+		CORBA_exception_free (&ev);
+	}
+
+	if (tool->config)
+		gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_GUI]);
+
+	return tool->config != NULL;
+}
+
 gboolean
 xst_tool_save (XstTool *tool)
 {
-	FILE *f;
-#warning This might be a security problem, fix
-	FILE *debug_file;
-	int fd_xml [2], fd_report [2];
-	int t;
+	CORBA_Environment ev;
+	ConfigArchiver_Archive archive;
+	ConfigArchiver_Location location;
+	gchar *backend_id;
 
 	g_return_val_if_fail (tool != NULL, FALSE);
 	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);
-	g_return_val_if_fail (tool->script_path, FALSE);
-
 	g_return_val_if_fail (root_access != ROOT_ACCESS_NONE, FALSE);
 
 	xst_dialog_freeze_visible (tool->main_dialog);
 
 	gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_XML]);
 
-	/* FIXME: Instead of doing the following, we should pass around a value describing
-	 * tool I/O mode (as opposed to a string to report_progress ()). */
-	tool->report_hook_type = XST_REPORT_HOOK_SAVE;
-
 #ifdef XST_DEBUG
 	/* don't actually save if we are just pretending */
 	if (root_access == ROOT_ACCESS_SIMULATED) {
@@ -547,49 +913,44 @@ xst_tool_save (XstTool *tool)
 	}
 #endif
 
-	pipe (fd_xml);
-	pipe (fd_report);
+	CORBA_exception_init (&ev);
 
-	t = fork ();
-	if (t < 0) {
-		g_error ("Unable to fork.");
-	} else if (t) {
-		/* Parent */
-
-		close (fd_xml [0]);	/* Close reading end of XML pipe */
-		close (fd_report [1]);  /* Close writing end of report pipe */
-
-		f = fdopen (fd_xml [1], "w");
-		debug_file = fopen ("/tmp/xst-fe-write", "w");
-
-		xmlDocDump (f, tool->config);
-		xmlDocDump (debug_file, tool->config);
-		
-		fclose (f);
-		fclose (debug_file);
-		
-		report_progress (tool, fd_report [0], _("Updating your system configuration."));
-		close (fd_report [0]);
+	/* Archive data with the archiver */
+	archive = bonobo_get_object ("archive:global-archive", "IDL:ConfigArchiver/Archive:1.0", &ev);
 
+	if (BONOBO_EX (&ev) || archive == CORBA_OBJECT_NIL) {
+		g_critical ("Could not resolve the archive moniker");
 	} else {
-		/* Child */
-
-		close (fd_xml [1]);	/* Close writing end of XML pipe */
-		close (fd_report [0]);  /* Close reading end of report pipe */
-		dup2 (fd_xml [0], STDIN_FILENO);
-		dup2 (fd_report [1], STDOUT_FILENO);
-
-		if (tool->current_platform)
-			execl (tool->script_path, tool->script_path,
-			       "--set", "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform), NULL);
+		if (location_id == NULL)
+			location = ConfigArchiver_Archive__get_currentLocation (archive, &ev);
 		else
-			execl (tool->script_path, tool->script_path, "--set", "--progress",
-			       "--report", NULL);
+			location = ConfigArchiver_Archive_getLocation (archive, location_id, &ev);
 
-		g_error ("Unable to run backend: %s", tool->script_path);
+		if (BONOBO_EX (&ev) || location == CORBA_OBJECT_NIL) {
+			g_critical ("Could not get location %s", location_id);
+			return FALSE;
+		}
+
+		backend_id = strrchr (tool->script_path, '/');
+
+		if (backend_id != NULL)
+			backend_id++;
+		else
+			backend_id = tool->script_path;
+
+		location_client_store_xml (location, backend_id, tool->config, ConfigArchiver_STORE_MASK_PREVIOUS, &ev);
+
+		bonobo_object_release_unref (location, &ev);
+		bonobo_object_release_unref (archive, &ev);
 	}
 
+	CORBA_exception_free (&ev);
+
+	if (location_id == NULL)
+		xst_tool_run_set_directive (tool, tool->config,
+					    _("Updating your system configuration."),
+					    "set", NULL);
+
 	xst_dialog_thaw_visible (tool->main_dialog);
 	return TRUE;  /* FIXME: Determine if it really worked. */
 }
@@ -600,105 +961,17 @@ xst_tool_save_cb (GtkWidget *w, XstTool *tool)
 	xst_tool_save (tool);
 }
 
-gboolean
-xst_tool_load (XstTool *tool)
-{
-	int fd [2];
-	int t, len;
-	char *p;
-
-	g_return_val_if_fail (tool != NULL, FALSE);
-	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);	
-	g_return_val_if_fail (tool->script_path, FALSE);
-  
-	/* FIXME: Instead of doing the following, we should pass around a value describing
-	 * tool I/O mode (as opposed to a string to report_progress ()). */
-	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
-
-	xmlSubstituteEntitiesDefault (TRUE);
-
-	if (tool->config) {
-		xmlFreeDoc (tool->config);
-		tool->config = NULL;
-	}
-
-	pipe (fd);
-			  
-	t = fork ();
-
-	if (t < 0) {
-		g_error ("Unable to fork.");
-	} else if (t) {
-		/* Parent */
-
-		close (fd [1]);	/* Close writing end */
-
-		/* LibXML support for parsing from memory is good, but parsing from
-		 * opened filehandles is not supported unless you write your own feed
-		 * mechanism. Let's just load it all into memory, then. Also, refusing
-		 * enormous documents can be considered a plus. </dystopic> */
-
-		report_progress (tool, fd [0], _("Scanning your system configuration."));
-
-		if (tool->run_again)
-			return TRUE;
-
-		p = g_malloc (102400);
-		fcntl(fd [0], F_SETFL, 0);  /* Let's block */
-		len = 0; 
-		for (len = 0; (t = read (fd [0], p + len, 102399 - len)); len += t)
-			;
-
-		if (len < 1 || len == 102399) {
-			g_free (p);
-		} else {
-			p = g_realloc (p, len + 1);
-			*(p + len) = 0;
-
-			tool->config = xmlParseDoc (p);
-			g_free (p);
-			close (fd [0]);
-		}
-	} else {
-		/* Child */
-
-		close (fd [0]);	/* Close reading end */
-		dup2 (fd [1], STDOUT_FILENO);
-
-		if (tool->current_platform)
-			execl (tool->script_path, tool->script_path,
-			       "--get", "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform), NULL);
-		else
-			execl (tool->script_path, tool->script_path, "--get", "--progress",
-			       "--report", NULL);
-
-		g_error ("Unable to run backend: %s", tool->script_path);
-	}
-	
-	if (tool->config)
-		gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_GUI]);
-
-	return tool->config != NULL;
-}
-
 void
 xst_tool_load_try (XstTool *tool)
 {
 	GtkWidget *d;
 
-	do
-	{
-		tool->run_again = FALSE;
-
-		if (!xst_tool_load (tool)) {
-			d = gnome_ok_dialog (_("There was an error running the backend script,\n"
-					       "and the configuration could not be loaded."));
-			gnome_dialog_run_and_close (GNOME_DIALOG (d));
-			exit (1);
-		}
+	if (!xst_tool_load (tool)) {
+		d = gnome_ok_dialog (_("There was an error running the backend script,\n"
+				       "and the configuration could not be loaded."));
+		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		exit (0);
 	}
-	while (tool->run_again);
 }
 
 void
@@ -796,6 +1069,8 @@ xst_tool_type_init (XstTool *tool)
 
 	tool->glade_common_path  = g_strdup_printf ("%s/common.glade", INTERFACES_DIR);
 
+	tool->etspecs_common_path = g_strdup (ETSPECS_DIR);
+
 	xml = xst_tool_load_glade_common (tool, "report_window");
 
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
@@ -879,7 +1154,11 @@ xst_tool_construct (XstTool *tool, const char *name, const char *title)
 			    tool);
 
 	tool->report_hook_list = NULL;
+	memset (&tool->report_hook_defaults, 0, sizeof (XstReportHook *) * XST_MAJOR_MAX);
 	xst_tool_add_report_hooks (tool, common_report_hooks);
+
+	tool->backend_pid = tool->backend_read_fd = tool->backend_write_fd = -1;
+	xst_tool_set_close_func (tool, xst_tool_kill_backend, NULL);
 }
 
 XstTool *
@@ -936,14 +1215,54 @@ xst_tool_add_report_hooks (XstTool *tool, XstReportHookEntry *report_hook_table)
 	}
 }
 
+void
+xst_tool_set_default_hook (XstTool *tool, XstReportHookEntry *entry, XstReportMajor major)
+{
+	g_return_if_fail (major < XST_MAJOR_MAX);
+	
+	if (tool->report_hook_defaults[major])
+		g_free (tool->report_hook_defaults[major]);
+
+	tool->report_hook_defaults[major] = xst_report_hook_new_from_entry (entry);
+}
+
+static gboolean
+xst_tool_call_report_hook (XstTool *tool, XstReportHook *hook, XstReportLine *rline)
+{
+	gboolean res;
+	
+	hook->invoked = TRUE;
+	tool->input_block = TRUE;
+	res = hook->func (tool, rline, hook->data);
+	tool->input_block = FALSE;
+	return res;
+}
+
+static void
+xst_tool_invoke_default_hook (XstTool *tool, XstReportLine *rline)
+{
+	XstReportHook *hook;
+	
+	if (tool->report_hook_defaults[rline->major]) {
+		hook = tool->report_hook_defaults[rline->major];
+		if ((hook->allow_repeat || !hook->invoked) &&
+		    (hook->type == XST_REPORT_HOOK_LOADSAVE ||
+		     hook->type == tool->report_hook_type)) {
+			xst_tool_call_report_hook (tool, hook, rline);
+		}
+	}
+}
+
 void
 xst_tool_invoke_report_hooks (XstTool *tool, XstReportHookType type, XstReportLine *rline)
 {
 	GSList *list;
 	XstReportHook *hook;
 	const gchar *key;
+	gboolean invoked;
 
 	key = xst_report_line_get_key (rline);
+	invoked = FALSE;
 
 	for (list = tool->report_hook_list; list; list = g_slist_next (list))
 	{
@@ -951,10 +1270,14 @@ xst_tool_invoke_report_hooks (XstTool *tool, XstReportHookType type, XstReportLi
 
 		if (!strcmp (hook->key, key) && (hook->allow_repeat || !hook->invoked) &&
 		    (hook->type == XST_REPORT_HOOK_LOADSAVE || hook->type == type)) {
-			hook->invoked = TRUE;
-			hook->func (tool, rline);
+			invoked = TRUE;
+			if (xst_tool_call_report_hook (tool, hook, rline))
+				return;
 		}
 	}
+
+	if (!invoked)
+		xst_tool_invoke_default_hook (tool, rline);
 }
 
 void
@@ -1008,16 +1331,75 @@ authenticate (gchar *exec_path)
 	}
 }
 
+static void
+try_show_usage_warning (void)
+{
+	gchar *key;
+	gboolean value;
+	gchar *warning = g_strdup_printf(_("Welcome to the %s prerelease of the "
+		  "Ximian Setup Tools.\n\n"
+		  "This is still a work in progress, and so it may have serious bugs.\n"
+		  "Due to the nature of these tools, bugs may render your computer\n"
+		  "PRACTICALLY USELESS, costing time, effort and sanity points.\n\n"
+		  "You have been warned. Thank you for trying out this prerelease of\n"
+		  "the Ximian Setup Tools!\n\n"
+		  "--\nThe Ximian Setup Tools team"), VERSION);
+
+	key = g_strjoin ("/", XST_CONF_ROOT, "global", "previously-run-" VERSION, NULL);
+
+	value = gnome_config_get_bool (key);
+
+	if (!value)
+	{
+		GnomeDialog *dialog;
+		GtkWidget *w0, *w1, *w2;
+
+		dialog = GNOME_DIALOG (gnome_warning_dialog (warning));
+		w0 = gtk_container_children (GTK_CONTAINER (GTK_BIN (GTK_BIN (dialog)->child)->child))->data;
+		w1 = GTK_WIDGET (gtk_check_button_new_with_label (_("Don't show me this again")));
+		gtk_widget_ref (w1);
+		w2 = GTK_WIDGET (gtk_alignment_new (1.0, 0.0, 0.0, 0.0));
+		gtk_container_add (GTK_CONTAINER (w2), w1);
+		gtk_box_pack_end (GTK_BOX (w0), w2, TRUE, TRUE, 0);
+		gtk_widget_show (w1);
+		gtk_widget_show (w2);
+
+		gnome_dialog_run_and_close (dialog);
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w1)))
+		{
+			gnome_config_set_bool (key, TRUE);
+			gnome_config_sync ();
+		}
+
+		gtk_widget_unref (w1);
+	}
+
+	g_free (warning);
+
+	g_free (key);
+}
+
 void
 xst_init (const gchar *app_name, int argc, char *argv [], const poptOption options)
 {
-	GtkWidget *d;
+	CORBA_ORB orb;
+
+	struct poptOption xst_options[] =
+	{
+		{ "location", '\0', POPT_ARG_STRING, &location_id, 0,
+		  N_("	Edit configuration in the location LOCATION."), N_("LOCATION") },
+		
+		{NULL, '\0', 0, NULL, 0}
+	};
 
 #ifdef ENABLE_NLS
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 #endif
 
+        gnomelib_register_popt_table (xst_options, "general XST options");
+
 	if (options == NULL) {
 		gnome_init (app_name, VERSION, argc, argv);
 	} else {
@@ -1039,6 +1421,11 @@ xst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 
 	glade_gnome_init ();
 
+	orb = oaf_init (argc, argv);
+	if (bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
+		g_critical ("Cannot initialize bonobo");
+	bonobo_activate ();
+
 	if (geteuid () == 0) {
 		root_access = ROOT_ACCESS_REAL;
 #ifdef XST_DEBUG
@@ -1050,4 +1437,6 @@ xst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 		authenticate (argv [0]);
 		root_access = ROOT_ACCESS_NONE;
 	}
+
+	try_show_usage_warning ();
 }
