@@ -45,6 +45,12 @@ struct _XstTool {
 	char *glade_path;
 	char *glade_common_path;
 	char *script_path;
+	char *etspecs_common_path;
+
+	/* backend process */
+	int backend_pid;
+	int backend_read_fd;
+	int backend_write_fd;
 
 	/* configuration */
 	xmlDoc  *config;
@@ -73,16 +79,19 @@ struct _XstTool {
 	gboolean timeout_done;
 	gboolean report_list_visible;
 
-	char *line;
-	int line_len;
-	guint input_id;
-	GSList *report_line_list;
-	gboolean report_dispatch_pending;
-	gboolean report_finished;
-	gboolean run_again;
+	GString *line;
+	GString *xml_document;
+	
+	guint     input_id;
+	gboolean  input_block;
+	GSList   *report_line_list;
+	gboolean  report_dispatch_pending;
+	gboolean  report_finished;
+	gboolean  run_again;
 
-	XstReportHookType report_hook_type;
-	GSList *report_hook_list;
+	XstReportHookType  report_hook_type;
+	GSList            *report_hook_list;
+	XstReportHook     *report_hook_defaults[XST_MAJOR_MAX];
 
 	XstPlatform *current_platform;     /* Always set from backend report */
 	GSList *supported_platforms_list;  /* Gets set only if backend breaks */
@@ -113,6 +122,15 @@ gboolean     xst_tool_save                (XstTool *tool);
 void         xst_tool_save_cb             (GtkWidget *w, XstTool *tool);
 gboolean     xst_tool_load                (XstTool *tool);
 void         xst_tool_load_try            (XstTool *tool);
+
+/* All undefined arguments in these directive functions must be (const gchar *) type.
+   They will become arguments in the directive line passed to the backend. Last arg must be NULL. */
+xmlDoc      *xst_tool_run_get_directive   (XstTool *tool, const gchar *report_sign,
+					   const gchar *directive, ...);
+/* xml can be NULL, in which case no XML will be sent to the backend. */
+xmlDoc      *xst_tool_run_set_directive   (XstTool *tool, xmlDoc *xml,
+					   const gchar *report_sign, const gchar *directive, ...);
+
 void         xst_tool_set_xml_funcs       (XstTool *tool, XstXmlFunc load_cb, XstXmlFunc save_cb, gpointer data);
 void         xst_tool_set_close_func      (XstTool *tool, XstCloseFunc close_cb, gpointer data);
 
@@ -122,6 +140,7 @@ GladeXML    *xst_tool_load_glade          (XstTool *tool, const gchar *widget);
 XstDialog   *xst_tool_get_dialog          (XstTool *tool);
 
 void         xst_tool_add_report_hooks    (XstTool *tool, XstReportHookEntry *report_hook_table);
+void         xst_tool_set_default_hook    (XstTool *tool, XstReportHookEntry *entry, XstReportMajor major);
 void         xst_tool_invoke_report_hooks (XstTool *tool, XstReportHookType type, XstReportLine *rline);
 void         xst_tool_reset_report_hooks  (XstTool *tool);
 
