@@ -99,6 +99,70 @@ on_network_notebook_switch_page (GtkWidget *notebook, GtkNotebookPage *page,
 	}
 }
 
+static gboolean
+is_ip_text_ok  (const gchar *text)
+{
+	gint i, dst, numdots;
+	IpVersion ver;
+
+	ver = IP_UNK;
+	dst = numdots = 0;
+
+	for (i = 0; text[i]; i++) {
+		/* IPv6 cases */
+		if (text[i] >= 'a' && text[i] <= 'f') {
+			if (ver == IP_V4)
+				return FALSE;
+			dst ++;
+			if (dst > 2)
+				return FALSE;
+			ver = IP_V6;
+			continue;
+		}
+
+		if (text[i] == ':') {
+			if (ver == IP_V4)
+				return FALSE;
+			dst = 0;
+			ver = IP_V6;
+			continue;
+		}
+
+		/* IPv4 cases */
+		if (text[i] == '.') {
+			if (ver == IP_V6)
+				return FALSE;
+			if (i == 0 || text[i - 1] == '.')
+				return FALSE;
+			numdots ++;
+			if (numdots > 3)
+				return FALSE;
+			dst = 0;
+			ver = IP_V4;
+			continue;
+		}
+
+		if ((text[i] >= '0') && (text[i] <= '9')) {
+			dst ++;
+			if (dst == 3) {
+				if (ver == IP_V6)
+					return FALSE;
+				ver = IP_V4;
+				continue;
+			}
+
+			if (dst > 3)
+				return FALSE;
+
+			continue;
+		}
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 is_char_ok (char c, EditableFilterRules rules)
 {
@@ -108,6 +172,29 @@ is_char_ok (char c, EditableFilterRules rules)
 		((rules & EF_ALLOW_TEXT) && (isalpha (c) || c == '_' || c == '-'));
 }
 
+static gchar *
+str_insert_text (const gchar *str, const gchar *text, gint length, gint pos)
+{
+	gchar *buff;
+	gint i, len;
+
+	len = strlen (str);
+	
+	g_assert (pos <= len);
+
+	buff = g_new0 (char, len + length + 1);
+
+	for (i = 0; i < pos; i++)
+		buff[i] = str[i];
+	for (i = 0; i < length; i++)
+		buff[i + pos] = text[i];
+	for (i = 0; i < len - pos; i++)
+		buff[i + pos + length] = str[i + pos];
+	buff[len + length] = 0;
+
+	return buff;
+}
+
 void
 filter_editable (GtkEditable *editable, const gchar *text, gint length,
 		 gint *pos, gpointer data)
@@ -116,6 +203,20 @@ filter_editable (GtkEditable *editable, const gchar *text, gint length,
 	char *s = NULL;
 	EditableFilterRules rules = GPOINTER_TO_INT (data);
 
+	if (rules & EF_ALLOW_IP) {
+		gboolean success;
+
+		s = str_insert_text (gtk_editable_get_chars (editable, 0, -1), text, length, *pos);
+		success = is_ip_text_ok (s);
+		g_free (s);
+		s = NULL;
+		
+		if (success)
+			goto text_changed_success;
+		else
+			goto text_changed_fail;
+	}
+	
 	/* thou shalt optimize for the common case */
 	if (length == 1) {
 		if (is_char_ok (*text, rules))
@@ -201,7 +302,7 @@ init_editable_filters (XstDialog *dialog)
 		{ "wins_ip",     EF_ALLOW_NONE },
 		{ "dns_list",    EF_ALLOW_ENTER },
 		{ "search_list", EF_ALLOW_ENTER | EF_ALLOW_TEXT },
-		{ "ip",          EF_ALLOW_NONE }, 
+		{ "ip",          EF_ALLOW_IP }, 
 		{ "alias",       EF_ALLOW_ENTER | EF_ALLOW_TEXT },
 		{ "domain",      EF_ALLOW_TEXT },
 		{ NULL,          EF_ALLOW_NONE }
@@ -271,7 +372,7 @@ void
 on_connection_add_clicked (GtkWidget *w, gpointer null)
 {
 	XstConnection *cxn;
-	GtkWidget *d, *ppp, *eth, *wvlan, *plip, *clist;
+	GtkWidget *d, *ppp, *eth, *wvlan, *plip, *irlan, *clist;
 	gint res, row;
 	XstConnectionType cxn_type;
 	
@@ -281,6 +382,7 @@ on_connection_add_clicked (GtkWidget *w, gpointer null)
 	eth   = xst_dialog_get_widget (tool->main_dialog, "connection_type_eth");
 	wvlan = xst_dialog_get_widget (tool->main_dialog, "connection_type_wvlan");
 	plip  = xst_dialog_get_widget (tool->main_dialog, "connection_type_plip");
+	irlan = xst_dialog_get_widget (tool->main_dialog, "connection_type_irlan");
 
 	/* ppp is the default for now */
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ppp), TRUE);
@@ -298,6 +400,8 @@ on_connection_add_clicked (GtkWidget *w, gpointer null)
 		cxn_type = XST_CONNECTION_WVLAN;
 	else if (GTK_TOGGLE_BUTTON (plip)->active)
 		cxn_type = XST_CONNECTION_PLIP;
+	else if (GTK_TOGGLE_BUTTON (irlan)->active)
+		cxn_type = XST_CONNECTION_IRLAN;
 	else
 		cxn_type = XST_CONNECTION_UNKNOWN;
 
@@ -336,6 +440,7 @@ on_connection_delete_clicked (GtkWidget *w, gpointer null)
 	if (res)
 		return;
 
+	connection_default_gw_remove (cxn->dev);
 	connection_free (cxn);
 	gtk_clist_remove (GTK_CLIST (clist), connection_row_selected);
 	xst_dialog_modify (tool->main_dialog);
@@ -363,6 +468,19 @@ on_connection_activate_clicked (GtkWidget *w, gpointer null)
 	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
 	cxn = gtk_clist_get_row_data (GTK_CLIST (clist), connection_row_selected);
 	connection_update_row_enabled (cxn, TRUE);
+
+	if (xst_dialog_get_modified (tool->main_dialog)) {
+		xst_tool_save (tool);
+		xst_dialog_set_modified (tool->main_dialog, FALSE);
+	} else {
+		gchar *sign, *file;
+		
+		file = (cxn->file)? cxn->file: cxn->dev;
+		sign = g_strdup_printf (_("Activating connection ``%s.''"), cxn->name);
+		xst_tool_run_set_directive (tool, NULL, sign, "enable_iface", file, "1", NULL);
+	
+		g_free (sign);
+	}
 }
 
 void
@@ -374,6 +492,18 @@ on_connection_deactivate_clicked (GtkWidget *w, gpointer null)
 	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
 	cxn = gtk_clist_get_row_data (GTK_CLIST (clist), connection_row_selected);
 	connection_update_row_enabled (cxn, FALSE);
+
+	if (xst_dialog_get_modified (tool->main_dialog))
+		xst_tool_save (tool);
+	else {
+		gchar *sign, *file;
+		
+		file = (cxn->file)? cxn->file: cxn->dev;
+		sign = g_strdup_printf (_("Deactivating connection ``%s.''"), cxn->name);
+		xst_tool_run_set_directive (tool, NULL, sign, "enable_iface", file, "0", NULL);
+	
+		g_free (sign);
+	}
 }
 
 void
@@ -416,6 +546,7 @@ callbacks_check_hostname_hook (XstDialog *dialog, gpointer data)
 	gchar *hostname_new;
 	xmlNode *root, *node;
 	GtkWidget *entry;
+	gint res;
 
 	root = xst_xml_doc_get_root (dialog->tool->config);
 	node = xst_xml_element_find_first (root, "hostname");
@@ -425,12 +556,14 @@ callbacks_check_hostname_hook (XstDialog *dialog, gpointer data)
 	entry = xst_dialog_get_widget (dialog, "hostname");
 	hostname_new = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	if (strcmp (hostname_new, hostname_old))
+	res = strcmp (hostname_new, hostname_old);
+	g_free (hostname_old);
+	
+	if (res)
 	{
 		gchar *text = _("The host name has changed. This will prevent you\n"
-				"from launching new applications,\n"
-				"and so you will have to log in again.\n\nContinue anyway?");
-		gint res;
+				"from launching new applications, and so you will\n"
+				"have to log in again.\n\nContinue anyway?");
 		GtkWidget *message;
 		
 		message = gnome_message_box_new (text, GNOME_MESSAGE_BOX_WARNING,
@@ -458,22 +591,32 @@ gboolean
 callbacks_update_connections_hook (XstDialog *dialog, gpointer data)
 {
 	GtkWidget *clist;
-	gint i;
-	XstConnection *cxn;
 
 	clist = xst_dialog_get_widget (dialog, "connection_list");
-	g_return_val_if_fail (GTK_IS_CLIST (clist), TRUE);
-
-	for (i = 0; i < GTK_CLIST (clist)->rows; i++) {
-		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
-		g_return_val_if_fail (cxn != NULL, TRUE);
-		connection_set_row_pixtext (clist, i, cxn->enabled ? _("Active") :
-					    _("Inactive"), cxn->enabled);
-	}
+	connection_update_clist_enabled_apply (clist);
 
 	return TRUE;
 }
 
+void
+callbacks_check_dialer (GtkWindow *window, XstTool *tool)
+{
+	gboolean has_dialer;
+	
+	has_dialer = (gboolean) gtk_object_get_data (GTK_OBJECT (tool),
+						     "dialinstalled");
+	if (!has_dialer)
+	{
+		gchar *text = _("wvdial could not be found on your system.\n"
+				"You need to install wvdial, or the PPP (modem)\n"
+				"connections will not activate.");
+		GtkWidget *message;
+		
+		message = gnome_warning_dialog_parented (text, window);
+		gnome_dialog_run_and_close (GNOME_DIALOG (message));
+	}
+}
+
 gboolean
 callbacks_check_dialer_hook (XstDialog *dialog, gpointer data)
 {
@@ -508,3 +651,95 @@ callbacks_check_dialer_hook (XstDialog *dialog, gpointer data)
 
 	return TRUE;
 }
+
+static gboolean
+callbacks_disabled_gatewaydev_warn (XstTool *tool, XstConnection *cxn, gboolean *ignore_enabled)
+{
+	gchar *text = _("The default gateway device is not activated. This\n"
+			"will prevent you from connecting to the Internet.\n"
+			"\nContinue anyway?");
+	gint res;
+	GtkWidget *message;
+	
+	message = gnome_message_box_new (text, GNOME_MESSAGE_BOX_WARNING,
+					 _("Activate connection"),
+					 GNOME_STOCK_BUTTON_OK,
+					 GNOME_STOCK_BUTTON_CANCEL,
+					 NULL);
+	gnome_dialog_set_parent (GNOME_DIALOG (message), GTK_WINDOW (tool->main_dialog));
+	res = gnome_dialog_run_and_close (GNOME_DIALOG (message));
+	
+	switch (res) {
+	case 0:
+		connection_default_gw_fix (cxn, XST_CONNECTION_ERROR_ENABLED);
+		return TRUE;
+	case 1:
+		*ignore_enabled = TRUE;
+		return TRUE;
+	case 2:
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+callbacks_check_manual_gatewaydev (XstTool *tool)
+{
+	XstConnection *cxn;
+	XstConnectionErrorType error;
+	gboolean ignore_enabled;
+
+	ignore_enabled = FALSE;
+	cxn = connection_default_gw_get_connection (tool);
+
+	while ((error = connection_default_gw_check_manual (cxn, ignore_enabled))
+	       != XST_CONNECTION_ERROR_NONE)
+	{
+		switch (error) {
+		case XST_CONNECTION_ERROR_ENABLED:
+			if (callbacks_disabled_gatewaydev_warn (tool, cxn, &ignore_enabled))
+				continue;
+			else
+				return FALSE;
+		case XST_CONNECTION_ERROR_PPP:
+			connection_default_gw_fix (cxn, error);
+			continue;
+		case XST_CONNECTION_ERROR_STATIC:
+		{
+			GtkWidget *dialog;
+			gchar *txt = _("The default gateway device is missing gateway\n"
+				       "information. Please provide this information to\n"
+				       "proceed, or choose another default gateway device.\n");
+			
+			dialog = gnome_error_dialog_parented (txt, GTK_WINDOW (tool->main_dialog));
+			gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+			return FALSE;
+		}
+		break;
+		case XST_CONNECTION_ERROR_NONE:
+		case XST_CONNECTION_ERROR_OTHER:
+		default:
+			g_warning ("callbacks_check_manual_gatewaydev: shouldn't be here.");
+			return TRUE;
+		}
+	}
+
+	connection_default_gw_set_manual (tool, cxn);
+	
+	return TRUE;
+}
+
+gboolean
+callbacks_check_gateway_hook (XstDialog *dialog, gpointer data)
+{
+	XstTool *tool;
+
+	tool = XST_TOOL (data);
+	if (!gtk_object_get_data (GTK_OBJECT (tool), "gwdevunsup") &&
+	    gtk_object_get_data (GTK_OBJECT (tool), "gatewaydev"))
+		return callbacks_check_manual_gatewaydev (tool);
+
+	connection_default_gw_set_auto (tool);
+	return TRUE;
+}
