@@ -388,10 +388,16 @@ connection_set_modified (XstConnection *cxn, gboolean state)
 }
 
 static void
-load_icon (const gchar *file, GdkPixmap **pixmap, GdkBitmap **mask)
+load_icon (GtkWidget *clist, const gchar *file, GdkPixmap **pixmap, GdkBitmap **mask)
 {
-	GdkPixbuf *pb, *pb2;
+	GdkPixbuf *new, *pb, *pb2;
+	GtkStyle *style;
 	char *path;
+	guint height;
+	guchar *data;
+
+	style = gtk_widget_get_style (clist);
+	height = (guint) ((style->font->ascent + style->font->descent) * 2);
 
 	path = g_concat_dir_and_file (PIXMAPS_DIR, file);
 	pb = gdk_pixbuf_new_from_file (path);
@@ -402,12 +408,17 @@ load_icon (const gchar *file, GdkPixmap **pixmap, GdkBitmap **mask)
 		return;
 	}
 	g_free (path);
-	
+
 	pb2 = gdk_pixbuf_scale_simple (pb, 16, 16, GDK_INTERP_BILINEAR);
 	gdk_pixbuf_unref (pb);
 	
-	gdk_pixbuf_render_pixmap_and_mask (pb2, pixmap, mask, 127);
-	gdk_pixbuf_unref (pb2);
+	new = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 16, height);
+	data = gdk_pixbuf_get_pixels (new);
+	memset (data, 0, height * gdk_pixbuf_get_rowstride (new));
+	gdk_pixbuf_copy_area (pb2, 0, 0, 16, 16, new, 0, 0);
+	
+	gdk_pixbuf_render_pixmap_and_mask (new, pixmap, mask, 127);
+	gdk_pixbuf_unref (new);
 }
 
 static gchar *
@@ -433,28 +444,33 @@ connection_clist_cmp (GtkCList *clist, gconstpointer p1, gconstpointer p2)
 }
 
 static void
-connection_init_clist (XstTool *tool)
+connection_init_clist (GtkWidget *clist)
 {
-	GtkWidget *clist;
+	GtkStyle *style;
 	
-	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
 	gtk_clist_set_compare_func (GTK_CLIST (clist), connection_clist_cmp);
+	style = gtk_widget_get_style (clist);
+	gtk_clist_set_row_height (GTK_CLIST (clist), (guint) ((style->font->ascent +
+	style->font->descent) * 2));
 }
 
 extern void
 connection_init_gui (XstTool *tool)
 {
+	GtkWidget *clist;
 	XstConnectionType i;
 
-	for (i = XST_CONNECTION_OTHER; i < XST_CONNECTION_LAST; i++)
-		load_icon (xst_iface_desc[i].icon, &mini_pm[i], &mini_mask[i]);
+	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
 
-	load_icon ("gnome-light-off.png" /* "connection-inactive.xpm" */,
+	for (i = XST_CONNECTION_OTHER; i < XST_CONNECTION_LAST; i++)
+		load_icon (clist, xst_iface_desc[i].icon, &mini_pm[i], &mini_mask[i]);
+
+	load_icon (clist, "gnome-light-off.png" /* "connection-inactive.xpm" */,
 		   &active_pm[0], &active_mask[0]);
-	load_icon ("gnome-light-on.png" /*"connection-active.xpm" */,
+	load_icon (clist, "gnome-light-on.png" /*"connection-active.xpm" */,
 		   &active_pm[1], &active_mask[1]);
 
-	connection_init_clist (tool);
+	connection_init_clist (clist);
 }
 
 void
@@ -852,8 +868,9 @@ connection_update_row (XstConnection *cxn)
 void
 connection_add_to_list (XstConnection *cxn, GtkWidget *clist)
 {
-	int row;
+	gint row;
 	char *text[3] = { "Error", NULL };
+	GtkStyle *style;
 
 	g_return_if_fail (GTK_IS_CLIST (clist));
 	g_return_if_fail (cxn != NULL);
@@ -861,6 +878,14 @@ connection_add_to_list (XstConnection *cxn, GtkWidget *clist)
 	row = gtk_clist_append (GTK_CLIST (clist), text);
 	gtk_clist_set_row_data (GTK_CLIST (clist), row, cxn);
 
+	style = gtk_widget_get_style (clist);
+	gtk_clist_set_shift (GTK_CLIST (clist), row, 0,
+			     (guint) ((style->font->ascent + style->font->descent) / 2), 0);
+	gtk_clist_set_shift (GTK_CLIST (clist), row, 1,
+			     (guint) ((style->font->ascent + style->font->descent) / 2), 0);
+	gtk_clist_set_shift (GTK_CLIST (clist), row, 2,
+			     (guint) ((style->font->ascent + style->font->descent) / 2), 0);
+
 	connection_default_gw_add (cxn->dev);
 	connection_update_row (cxn);
 	connection_update_row_enabled (cxn, cxn->enabled);
@@ -1049,7 +1074,8 @@ connection_new_from_node (xmlNode *node)
 	
 	s = xst_xml_get_child_content (node, "bootproto");
 	if (s) {
-		cxn->ip_config = cxn->tmp_ip_config = connection_config_type_from_str (s);
+		cxn->ip_config = connection_config_type_from_str (s);
+		cxn->tmp_ip_config = cxn->ip_config;
 		g_free (s);
 	}
 
@@ -1165,9 +1191,6 @@ connection_check_netmask_gui (XstConnection *cxn)
         address = gtk_editable_get_chars (GTK_EDITABLE (address_widget), 0, -1);
         netmask = gtk_editable_get_chars (GTK_EDITABLE (netmask_widget), 0, -1);
 
-        g_print ("ip: %s\n", address);
-        g_print ("netmask: %s\n", netmask);
-
         if ((sscanf (address, "%d.", &ip1) == 1) && (!strlen (netmask))) {
 		if (ip1 < 127) {
 			gtk_entry_set_text (GTK_ENTRY (netmask_widget), "255.0.0.0");
@@ -1259,7 +1282,7 @@ static void
 empty_ip (XstConnection *cxn)
 {
 	connection_check_netmask_gui (cxn);
-	
+
 	cxn->ip_config = cxn->tmp_ip_config;
 	GET_BOOL ("ip_", update_dns);
 	GET_STR ("ip_", address);
@@ -1508,7 +1531,7 @@ static void
 update_ip_config (XstConnection *cxn)
 {
 	IPConfigType ip;
-
+	
 	ip = cxn->tmp_ip_config;
 
 	SET_BOOL ("ip_", update_dns);
@@ -1532,7 +1555,7 @@ ip_config_menu_cb (GtkWidget *w, gpointer data)
 	if (cxn->tmp_ip_config == ip)
 		return;
 
-	cxn->tmp_ip_config = GPOINTER_TO_INT (data);
+	cxn->tmp_ip_config = ip;
 
 	connection_set_modified (cxn, TRUE);
 
@@ -1666,10 +1689,74 @@ hookup_callbacks (XstConnection *cxn)
 		connect_editable_filter (W (s[i].name), s[i].rule);
 }
 
+
+/**
+ * connection_dialog_set_visible_pages:
+ * @cxn: 
+ * 
+ * Hides and shows the pages from the connection dialog notebook according to the
+ * connection type. It also sets the the icons for the pages depending on the
+ * connection type.
+ **/
+static void
+connection_dialog_set_visible_pages (XstConnection *cxn)
+{
+	GtkNotebook *nb;
+	
+	nb = GTK_NOTEBOOK (W ("connection_nb"));
+
+	if (cxn->type == XST_CONNECTION_PPP) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/ppp.png");
+		fill_ppp (cxn);
+		fill_ppp_adv (cxn);
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("ip_vbox")));
+		callbacks_check_dialer (GTK_WINDOW (cxn->window), tool);
+	} else {
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("ppp_vbox")));
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("ppp_adv_vbox")));
+	}
+       
+	if (cxn->type == XST_CONNECTION_WVLAN) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/wavelan-48.png");
+		fill_wvlan (cxn);
+		/* FIXME: temprorarily disabling this notebook, until we get support for this. */
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("wvlan_vbox")));
+	} else
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("wvlan_vbox")));
+
+	if (cxn->type == XST_CONNECTION_PLIP) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/plip-48.png");
+		fill_ptp (cxn);
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("ip_vbox")));
+	} else 
+		gtk_notebook_remove_page (nb,
+					  gtk_notebook_page_num (nb,
+								 W ("ptp_vbox")));
+
+	if (cxn->type == XST_CONNECTION_ETH) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/connection-ethernet.png");
+	}
+
+	if (cxn->type == XST_CONNECTION_IRLAN) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/irda-48.png");
+	}
+}
+
 void
 connection_configure (XstConnection *cxn)
 {
-	GtkWidget *nb;
 	gchar *s;
 
 	if (cxn->window) {
@@ -1685,10 +1772,12 @@ connection_configure (XstConnection *cxn)
 	cxn->xml = glade_xml_new (s, "connection_config_dialog");
 
 	g_assert (cxn->xml);
+	g_free (s);
 
 	hookup_callbacks (cxn);
 
 	cxn->window = W("connection_config_dialog");
+	cxn->tmp_ip_config = cxn->ip_config;
 
 	fill_general (cxn);
 	fill_ip      (cxn);
@@ -1698,54 +1787,7 @@ connection_configure (XstConnection *cxn)
 	if (!xst_xml_element_get_boolean (cxn->node->parent, "userifacfectl"))
 		gtk_widget_hide (W("status_user"));
 
-	/* would like to do this as a switch */
-	nb = W ("connection_nb");
-	if (cxn->type == XST_CONNECTION_PPP) {
-		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/ppp.png");
-		fill_ppp (cxn);
-		fill_ppp_adv (cxn);
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("ip_vbox")));
-	} else {
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("ppp_vbox")));
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("ppp_adv_vbox")));
-	}
-       
-	if (cxn->type == XST_CONNECTION_WVLAN) {
-		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/wavelan-48.png");
-		fill_wvlan (cxn);
-		/* FIXME: temprorarily disabling this notebook, until we get support for this. */
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("wvlan_vbox")));
-	} else
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("wvlan_vbox")));
-
-	if (cxn->type == XST_CONNECTION_PLIP) {
-		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/plip-48.png");
-		fill_ptp (cxn);
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("ip_vbox")));
-	} else 
-		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
-					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-								 W ("ptp_vbox")));
-
-	if (cxn->type == XST_CONNECTION_ETH) {
-		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/connection-ethernet.png");
-	}
-
-	if (cxn->type == XST_CONNECTION_IRLAN) {
-		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/irda-48.png");
-	}
+	connection_dialog_set_visible_pages (cxn);
 
 	cxn->frozen = FALSE;
 
