@@ -27,6 +27,7 @@
 
 #include "tables.h"
 #include "xml.h"
+#include "validate.h"
 #include "export-clients.h"
 
 extern XstTool *tool;
@@ -42,21 +43,6 @@ typedef struct
 }
 ExportClientsWidgets;
 
-/**
- * set_text_or_blank: Sets a GtkEntry to the supplied text value, which can be NULL.
- *
- * If the text reference is NULL, the entry is set to an empty string.
- */
-
-static void
-set_text_or_blank (GtkEntry *entry, const gchar *text)
-{
-	if (text)
-		gtk_entry_set_text (entry, text);
-	else
-		gtk_entry_set_text (entry, "");
-}
-
 static void
 export_clients_get_widgets (ExportClientsWidgets *w)
 {
@@ -70,7 +56,7 @@ export_clients_clear_entries (void)
 	ExportClientsWidgets w;
 
 	export_clients_get_widgets (&w);
-	set_text_or_blank            (w.pattern, NULL);
+	xst_ui_entry_set_text        (w.pattern, NULL);
 	gtk_toggle_button_set_active (w.write, FALSE);
 
 	export_clients_apply_policies ();
@@ -90,7 +76,7 @@ export_clients_from_client (ExportClient *client)
 
 	export_clients_get_widgets (&w);
 
-	set_text_or_blank            (w.pattern, client->pattern);
+	xst_ui_entry_set_text        (w.pattern, client->pattern);
 	gtk_toggle_button_set_active (w.write,   client->write);
 }
 
@@ -161,10 +147,21 @@ export_clients_run (void)
 	gint ret;
 
 	clients_dialog = xst_dialog_get_widget (tool->main_dialog, "export_clients");
-
+	export_clients_apply_policies ();
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (clients_dialog));
 }
 
+static gboolean
+validate_client (ExportClient *client)
+{
+	g_return_val_if_fail (client != NULL, FALSE);
+
+	if (!validate_export_client_pattern (client->pattern))
+		return FALSE;
+
+	return TRUE;
+}
+
 void
 export_client_add_cb (void)
 {
@@ -172,6 +169,12 @@ export_client_add_cb (void)
 
 	client = g_new0 (ExportClient, 1);
 	export_clients_to_client (client);
+	if (!validate_client (client))
+	{
+		g_free (client);
+		return;
+	}
+
 	export_clients_table_insert (client);
 	export_clients_clear_entries ();
 }
@@ -187,6 +190,7 @@ void
 export_client_delete_cb (void)
 {
 	export_clients_table_delete_selected ();
+	export_clients_clear_entries ();
 }
 
 void
