@@ -30,6 +30,8 @@
 #include <gal/e-table/e-cell-text.h>
 #include <gal/e-paned/e-hpaned.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include "xst-shares-tool.h"
+#include "xst-mount.h"
 #include "tables.h"
 #include "xml.h"
 #include "validate.h"
@@ -47,7 +49,7 @@
 
 typedef gboolean  (XstValidatorFunc)(gpointer val);
 typedef gchar    *(XstFilterFunc)   (gchar *val);
-typedef void      (XstChangedFunc)  (gpointer val);
+typedef gboolean  (XstChangedFunc)  (gpointer val);
 typedef gpointer  (XstValueFunc)    (xmlNodePtr node);
 
 typedef enum
@@ -70,10 +72,11 @@ typedef struct
 }
 XstTableColDef;
 
-static void     set_mount_button_label_neg    (gpointer unmount);
-static void     mount_button_from_import_node (xmlNodePtr node);
-static void     export_set_writable           (gpointer writable);
-static gpointer export_get_writable           (xmlNodePtr node);
+static gboolean mount_state_changed_from_table (gpointer unmount);
+static void     set_mount_button_label_neg     (gpointer unmount);
+static void     mount_button_from_import_node  (xmlNodePtr node);
+static gboolean export_set_writable            (gpointer writable);
+static gpointer export_get_writable            (xmlNodePtr node);
 
 extern XstTool *tool;
 
@@ -116,7 +119,7 @@ XstTableColDef imports_table_col_def [] =
 	{ "host",    XST_TABLE_COL_STRING, xst_util_nice_hostname_dup, validate_import_host_name,   NULL, NULL },
 	{ "path",    XST_TABLE_COL_STRING, xst_util_nice_path_dup,     validate_import_remote_path, NULL, NULL },
 	{ "point",   XST_TABLE_COL_STRING, xst_util_nice_path_dup,     validate_import_local_path,  NULL, NULL },
-	{ "mounted", XST_TABLE_COL_STATE,  NULL,                       NULL,                        set_mount_button_label_neg, NULL },
+	{ "mounted", XST_TABLE_COL_STATE,  NULL,                       NULL,                        mount_state_changed_from_table, NULL },
 	{ NULL }
 };
 
@@ -175,6 +178,141 @@ XstWidgetUserPolicy export_clients_unselected_policies[] =
 
 static gchar *blank_string = "";
 
+static gchar *
+print_mount_list (GList *list)
+{
+	gchar *text = NULL;
+
+	for ( ; list; list = g_list_next (list)) {
+		XstMount *mount = list->data;
+		gchar *s0, *s1;
+
+		s0 = g_strdup_printf (_("%s on %s\n"), mount->device, mount->point);
+		if (text) {
+			s1 = g_strconcat (text, s0, NULL);
+			g_free (text);
+			text = s1;
+		} else {
+			text = g_strdup (s0);
+		}
+		g_free (s0);
+	}
+
+	return text;
+}
+
+static void
+set_mount_list_mounted (XstTool *tool, GList *list, gboolean state)
+{
+	xst_dialog_freeze (tool->main_dialog);
+
+	for ( ; list; list = g_list_next (list)) {
+		XstMount *mount = list->data;
+		xmlNodePtr node;
+
+		node = shares_xml_find_import_node_by_point (mount->point);
+		if (!node) {
+			g_warning ("Could not match mount point: %s", mount->point);
+			continue;
+		}
+
+		xst_xml_element_set_state (node, "mounted", state);
+	}
+
+	xst_dialog_thaw (tool->main_dialog);
+}
+
+static void
+free_mount_list (GList *list)
+{
+	for ( ; list; list = g_list_next (list)) {
+		xst_mount_free (list->data);
+	}
+
+	g_list_free (list);
+}
+
+void
+check_mount_change (XstSharesTool *tool)
+{
+	gchar *message = NULL, *s0, *s1;
+	GtkWidget *dialog;
+	xmlNodePtr node;
+
+	xst_dialog_freeze (XST_TOOL (tool)->main_dialog);
+
+	if (tool->failed_mounts) {
+		gchar *head_text = g_list_next (tool->failed_mounts) ?
+			_("Could not mount the following shares -\n"
+			  "they could be temporarily unavailable, or you\n"
+			  "might have specified bad parameters:\n\n") :
+			_("Could not mount the following share -\n"
+			  "it could be temporarily unavailable, or you\n"
+			  "might have specified bad parameters:\n\n");
+
+		s0 = print_mount_list (tool->failed_mounts);
+		message = g_strconcat (head_text, s0, NULL);
+		g_free (s0);
+
+		set_mount_list_mounted (tool, tool->failed_mounts, FALSE);
+		free_mount_list (tool->failed_mounts);
+		tool->failed_mounts = NULL;
+	}
+
+	if (tool->failed_unmounts) {
+		if (message) {
+			gchar *head_text = g_list_next (tool->failed_unmounts) ?
+				_("\nAlso, could not unmount the following shares -\n"
+				  "they might be in use by programs:\n\n") :
+				_("\nAlso, could not unmount the following share -\n"
+				  "it might be in use by programs:\n\n");
+
+			s0 = g_strconcat (message, head_text, NULL);
+			g_free (message);
+			message = s0;
+		} else {
+			gchar *head_text = g_list_next (tool->failed_unmounts) ?
+				_("Could not unmount the following shares -\n"
+				  "they might be in use by programs:\n\n") :
+				_("Could not unmount the following share -\n"
+				  "it might be in use by programs:\n\n");
+
+			message = g_strdup (head_text);
+		}
+
+		s0 = print_mount_list (tool->failed_unmounts);
+		s1 = g_strconcat (message, s0, NULL);
+		g_free (message);
+		g_free (s0);
+		message = s1;
+
+		set_mount_list_mounted (tool, tool->failed_unmounts, TRUE);
+		free_mount_list (tool->failed_unmounts);
+		tool->failed_unmounts = NULL;
+	}
+
+	node = import_table_get_selected_node ();
+	if (node)
+		mount_button_from_import_node (node);
+
+	if (message) {
+		dialog = gnome_error_dialog_parented (message, GTK_WINDOW (XST_TOOL (tool)->main_dialog));
+		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+
+		g_free (message);
+	}
+
+	xst_dialog_thaw (XST_TOOL (tool)->main_dialog);
+	xst_dialog_set_modified (XST_TOOL (tool)->main_dialog, FALSE);
+}
+
+static void
+apply_mount_change (XstSharesTool *tool)
+{
+	xst_tool_save (XST_TOOL (tool));
+	check_mount_change (tool);
+}
+
 static void
 set_value_at_with_col_def (ETableModel *etm, int col, int row, const void *val, void *data)
 {
@@ -223,9 +361,12 @@ set_value_at_with_col_def (ETableModel *etm, int col, int row, const void *val,
 	}
 
 	if (def->changed)
-		def->changed (value);
-
-	xst_dialog_modify (tool->main_dialog);
+	{
+		if (def->changed (value))
+			xst_dialog_modify (tool->main_dialog);
+	}
+	else
+		xst_dialog_modify (tool->main_dialog);
 }
 
 static gpointer
@@ -748,6 +889,16 @@ set_mount_button_label_neg (gpointer unmount)
 	set_mount_button_label (!((gboolean) unmount));
 }
 
+static gboolean
+mount_state_changed_from_table (gpointer unmount)
+{
+	set_mount_button_label_neg (unmount);
+
+	/* Save changes */
+	apply_mount_change (XST_SHARES_TOOL (tool));
+	return FALSE;
+}
+
 static void
 mount_button_from_import_node (xmlNodePtr node)
 {
@@ -839,6 +990,8 @@ import_table_mount_selected_node_toggle_cb (void)
 	xmlNodePtr node;
 	gboolean state;
 
+	xst_dialog_freeze (tool->main_dialog);
+
 	table = e_table_scrolled_get_table (E_TABLE_SCROLLED (imports_table_scrolled));
 	model = table->model;
 
@@ -849,17 +1002,23 @@ import_table_mount_selected_node_toggle_cb (void)
 	xst_xml_element_set_state (node, "mounted", !state);
 
 	import_table_update_selected ();
+
+	/* Save changes */
+	apply_mount_change (XST_SHARES_TOOL (tool));
+
+	xst_dialog_thaw (tool->main_dialog);
 }
 
 /* --- */
 
-static void
+static gboolean
 export_set_writable (gpointer writable)
 {
 	xmlNodePtr node;
 
 	node = export_table_get_selected_node ();
 	shares_xml_export_set_general_write (node, (gboolean) writable);
+	return TRUE;
 }
 
 static gpointer
@@ -1072,3 +1231,25 @@ export_clients_table_from_export_node (xmlNodePtr export_node)
 		g_slist_free (rowdata);
 	}
 }
+
+gboolean
+import_mount_failed (XstSharesTool *tool, XstReportLine *rline, gpointer data)
+{
+	XstMount *mount;
+
+	mount = xst_mount_new (rline->argv [0], rline->argv [1]);
+	tool->failed_mounts = g_list_append (tool->failed_mounts, mount);
+
+	return TRUE;
+}
+
+gboolean
+import_unmount_failed (XstSharesTool *tool, XstReportLine *rline, gpointer data)
+{
+	XstMount *mount;
+
+	mount = xst_mount_new (rline->argv [0], rline->argv [1]);
+	tool->failed_unmounts = g_list_append (tool->failed_unmounts, mount);
+
+	return TRUE;
+}
