@@ -21,10 +21,20 @@
  * Authors: Tambet Ingo <tambet@ximian.com> and Arturo Espinosa <arturo@ximian.com>.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <ctype.h>
 #include <gnome.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <errno.h>
+#include <utmp.h>
+
 #include "callbacks.h"
 #include "user_group.h"
 #include "e-table.h"
@@ -41,10 +51,8 @@ static int reply;
 /* Static prototypes */
 
 static void group_settings_prepare (ug_data *ud);
-static void group_update_users (xmlNodePtr node, gchar *old_name, gchar *new_name);
 static void group_update_xml (xmlNodePtr node, gboolean adv);
 static GList *get_group_users (xmlNodePtr group_node);
-static GList *get_group_mainusers (xmlNodePtr group_node);
 static GList *group_fill_members_list (xmlNodePtr node);
 static void group_fill_all_users_list (xmlNodePtr node, GList *exclude);
 static void del_group_users (xmlNodePtr group_node);
@@ -193,7 +201,7 @@ user_filter (xmlNodePtr node)
 }
 
 gboolean
-check_node_complexity (xmlNodePtr node)
+check_node_visibility (xmlNodePtr node)
 {
 	xmlNodePtr db_node;
 	gchar *field, *content;
@@ -240,21 +248,28 @@ check_node_complexity (xmlNodePtr node)
 }
 
 static gboolean
-is_valid_name (const gchar *str)
+is_valid_name (const gchar *name)
 {
-	if (!str || !*str)
-		return FALSE;
+	/*
+	 * User/group names must start with a letter, and may not
+	 * contain colons, commas, newlines (used in passwd/group
+	 * files...) or any non-printable characters.
+	 */
+        if (!*name || !isalpha(*name))
+                return FALSE;
 
-	for (;*str; str++) {
-		if (((*str < 'a') || (*str > 'z')) &&
-				((*str < '0') || (*str > '9')))
+        while (*name) {
+                if (*name == ':' || *name == ',' ||
+                    *name == '\n' || !isprint(*name))
+                        return FALSE;
 
-			return FALSE;
-	}
+                name++;
+        }
 
-	return TRUE;
+        return TRUE;
 }
 
+
 static gboolean
 is_valid_id (const gchar *str)
 {
@@ -314,6 +329,7 @@ gchar *
 check_user_login (xmlNodePtr node, const gchar *login)
 {
 	gchar *buf = NULL;
+	struct utmp ut;
 
 	g_return_val_if_fail (node != NULL, FALSE);
 	g_return_val_if_fail (login != NULL, FALSE);
@@ -323,7 +339,7 @@ check_user_login (xmlNodePtr node, const gchar *login)
 		buf = g_strdup (_("The username is empty."));
 
 	/* If too long. */
-	else if (strlen (login) > 32)
+	else if (strlen (login) > sizeof (ut.ut_user))
 		buf = g_strdup (_("The username is too long."));
 
 	else if (!check_user_root (node, "login", login))
@@ -335,7 +351,7 @@ check_user_login (xmlNodePtr node, const gchar *login)
 
 	/* if !exist. */
 	else if (node_exists (node, "login", login))
-		buf = g_strdup (_("Username already exists."));
+		buf = g_strdup_printf (_("Username \"%s\" already exists.\n\nPlease select a different Username"), login);
 
 	return buf;
 }
@@ -356,11 +372,31 @@ check_user_uid (xmlNodePtr node, const gchar *val)
 	return buf;
 }
 
-gchar *
-check_user_comment (xmlNodePtr node, const gchar *val)
+static gchar *
+user_account_check_comment (UserAccount *account)
 {
-	/* What could be wrong with comment? */
-	return NULL;
+	gint i;
+	gchar *buf = NULL;	
+	gchar *comment;
+
+	/* comment can be empty, no error */
+	if (!account->comment)
+		return buf;
+
+	comment = g_strjoinv (NULL, account->comment);
+	
+	for (i = 0; i < strlen (comment); i++) {
+		if (iscntrl (comment[i]) || comment[i] == ',' ||
+		    comment[i] == '=' || comment[i] == ':') {
+
+			buf = g_strdup_printf (N_("Invalid character '%c' in comment."),
+						  comment[i]);
+			break;
+		}
+	}
+
+	g_free (comment);
+	return buf;
 }
 
 gchar *
@@ -398,19 +434,33 @@ parse_home (UserAccount *account)
 	return home;
 }
 
-gchar *
-check_user_home (xmlNodePtr node, const gchar *val)
+static gchar *
+user_account_check_home (UserAccount *account)
 {
-	gchar *buf = NULL;
+	gchar *home = account->home;
+	xmlNodePtr node = account->node;
+	struct stat s;
+	
+	if (!home || (strlen (home) < 1))
+		return g_strdup (N_("Home directory must not be empty."));
 
-	if (strlen (val) < 1)
-		buf = g_strdup (_("Home directory must not be empty."));
-	else if (*val != '/')
-		buf = g_strdup (_("Please enter full path for home directory."));
-	else if (!check_user_root (node, "home", val))
-		buf = g_strdup (_("root user shouldn't be modified."));
+	if (*home != '/')
+		return g_strdup (N_("Please enter full path for home directory."));
 
-	return buf;
+	if (!check_user_root (node, "home", home))
+		return g_strdup (N_("root user shouldn't be modified."));
+
+	if (stat (home, &s))
+	{
+		switch (errno) {
+		case ENOTDIR: return _("Part of the path to the home directory is a file.");         
+		case ELOOP:   return _("There is a loop in the path to the home directory.");        
+		case ENOMEM:  return _("Not enough memory to check the home directory.");            
+		case ENAMETOOLONG: return _("The path to the home directory is too long.");          
+		}
+	}
+
+	return NULL;
 }
 
 static gchar *
@@ -426,20 +476,21 @@ parse_group (UserAccount *account, const gchar *val)
 	return buf;
 }
 
-gint
-check_user_group (UserAccount *account, const gchar *val, gchar **error)
+static gint
+user_account_check_group (UserAccount *account, gchar **error)
 {
 	xmlNodePtr group_node;
 	gchar *group;
 	gint retval;
-
-	group = parse_group (account, val);
+	gchar *group_name = account->group;	
+	
+	group = parse_group (account, group_name);
 	group_node = get_corresp_field (account->node);
 
 	if (!is_valid_name (group)) {
 		*error = g_strdup (_("Group name is not valid."));
 		retval = -1;
-	} else if (node_exists (group_node, "group", val)) {
+	} else if (node_exists (group_node, "group", group_name)) {
 		*error = g_strdup (_("Such group id already exists."));
 		retval = 0;
 	} else {
@@ -620,7 +671,7 @@ get_user_list (gchar *field, xmlNodePtr group_node)
 	     u = xst_xml_element_find_next (u, "user"))
 	{
 
-		if (check_node_complexity (u))
+		if (check_node_visibility (u))
 			list = g_list_prepend (list, xst_xml_get_child_content (u, field));
 	}
 
@@ -933,7 +984,7 @@ group_settings_prepare (ug_data *ud)
 
 	w0 = xst_dialog_get_widget (tool->main_dialog, "group_settings_name");
 	gtk_widget_set_sensitive (w0, xst_tool_get_access (tool));
-	my_gtk_entry_set_text (w0, name);
+	xst_ui_entry_set_text (w0, name);
 
 	/* Fill group members */
 	member_rows = group_fill_members_list (ud->node);
@@ -996,6 +1047,8 @@ group_update_xml (xmlNodePtr node, gboolean adv)
 		add_group_users (node, buf);
 }
 
+#if 0
+/* Not used at the moment, if'ed out to get rid of complier warning. */
 static void
 group_update_users (xmlNodePtr node, gchar *old_name, gchar *new_name)
 {
@@ -1035,6 +1088,7 @@ group_update_users (xmlNodePtr node, gchar *old_name, gchar *new_name)
 		}
 	}
 }
+#endif
 
 static GList *
 get_group_users (xmlNodePtr group_node)
@@ -1057,6 +1111,8 @@ get_group_users (xmlNodePtr group_node)
 	return userlist;
 }
 
+#if 0
+/* Not used at the moment, if'ed out to get rid of complier warning. */
 static GList *
 get_group_mainusers (xmlNodePtr group_node)
 {
@@ -1088,6 +1144,7 @@ get_group_mainusers (xmlNodePtr group_node)
 	g_free (gid);
 	return userlist;
 }
+#endif
 
 static GList *
 group_fill_members_list (xmlNodePtr node)
@@ -1192,22 +1249,28 @@ user_query_string_get (void)
 
 
 UserAccount *
-user_account_get_default (void)
+user_account_new (const gchar *profile)
 {
 	UserAccount *account;
 	xmlNodePtr node;
 	gchar *uid;
-	Profile *pf = profile_table_get_profile (NULL);
+	Profile *pf = profile_table_get_profile (profile);
+
+	if (!pf) {
+		g_warning ("user_account_new: Couldn't get profile %s. Using default.",
+			   profile);
+		pf = profile_table_get_profile (NULL);
+	}
 
 	node = get_user_root_node ();
 	if (!node) {
-		g_warning ("user_account_get_default: Couldn't get node for user.");
+		g_warning ("user_account_new: Couldn't get node for user.");
 		return NULL;
 	}
 
 	uid = find_new_id (node);
 	if (!uid) {
-		g_warning ("user_account_get_default: Couldn't get uid for user.");
+		g_warning ("user_account_new: Couldn't get uid for user.");
 		return NULL;
 	}
 	
@@ -1239,7 +1302,7 @@ user_account_get_by_node (xmlNodePtr node)
 	account->new = FALSE;
 
 	account->name = user_value_login (node);
-	account->comment = user_value_comment (node);
+	account->comment = user_value_comment_array (node);
 	account->uid = user_value_uid_string (node);
 	account->home = user_value_home (node);
 	account->shell = user_value_shell (node);
@@ -1261,11 +1324,13 @@ user_account_save (UserAccount *account)
 	gchar *buf;
 	xmlNodePtr node = account->node;
 
+	account->modified = FALSE;
+
 	if (account->new)
 		node = account->node = user_add_blank_xml (account->node);
 	
 	user_set_value_login (node, account->name);
-	user_set_value_comment (node, account->comment);
+	user_set_value_comment_array (node, account->comment);
 	user_set_value_uid_string (node, account->uid);
 	user_set_value_shell (node, account->shell);
 
@@ -1290,11 +1355,123 @@ user_account_save (UserAccount *account)
 	xst_dialog_modify (tool->main_dialog);
 }
 
+gchar *
+user_account_check (UserAccount *account)
+{
+	gchar *buf;
+	xmlNodePtr node = account->node;	
+
+	if ((buf = check_user_login (node, account->name)))
+		return buf;
+
+	if ((buf = user_account_check_comment (account)))
+		return buf;
+	
+	if ((buf = user_account_check_home (account)))
+		return buf;
+
+	if ((buf = check_user_shell (node, account->shell)))
+		return buf;
+
+	if ((buf = check_user_uid (node, account->uid)))
+		return buf;
+
+	if ((user_account_check_group (account, &buf)) < 0)
+		return buf;
+			
+	return NULL;
+}
+
+static GList *
+user_account_check_home_warnings (UserAccount *account, GList *warnings)
+{
+	gchar *home;
+	struct stat s;
+
+	home = parse_home (account);
+
+	if (stat (home, &s))
+	{
+		gchar *text = NULL;
+		
+		switch (errno) {
+		case ENOENT:
+			if (!account->new)
+				text = _("The home directory does not exist, or its path is invalid.");
+			break;
+		case EACCES:
+			if (!account->new)
+				text = _("Couldn't get access to the home directory.");
+			break;
+		default:
+			if (!account->new)
+				/* We shouldn't fall here: the other cases are in the error checks. */
+				text = _("There was an error trying to access the home directory.");
+		}
+
+		if (text)
+			warnings = g_list_append (warnings, text);
+	} else {
+		if (account->new) {
+			warnings = g_list_append
+				(warnings, _("The home directory already exists. "
+					     "This will break the new user's permissions."));
+		} else {
+			if (!S_ISDIR (s.st_mode))
+				warnings = g_list_append
+					(warnings, _("The home directory path exists, "
+						     "but it is not a directory."));
+			if (account->uid && (s.st_uid != atoi (account->uid)))
+				warnings = g_list_append
+					(warnings, _("The home directory is not owned by the user."));
+			else {
+				if (!s.st_mode & S_IRUSR)
+					warnings = g_list_append
+						(warnings, _("The user doesn't have permission to "
+							     "read from the home directory."));
+				if (!s.st_mode & S_IWUSR)
+					warnings = g_list_append
+						(warnings, _("The user doesn't have permission to "
+							     "write in the home directory."));
+				if (!s.st_mode & S_IRUSR)
+					warnings = g_list_append
+						(warnings, _("The user doesn't have permission to "
+							     "use (``execute'') the home directory."));
+			}
+		}
+	}
+
+	g_free (home);
+		
+	return warnings;
+}
+
+GList *
+user_account_check_warnings (UserAccount *account)
+{
+	GList *warnings = NULL;
+
+	warnings = user_account_check_home_warnings (account, warnings);
+
+	return warnings;
+}
+
+void
+user_account_destroy_warnings (GList *warnings)
+{
+	GList *l, *l2;
+
+	for (l2 = l = warnings; l2; l = l2) {
+		l2 = l->next;
+		g_list_remove_link (warnings, l);
+	}
+}
+
 void
 user_account_destroy (UserAccount *account)
 {
 	if (account->name)     g_free (account->name);
-	if (account->comment)  g_free (account->comment);
+	if (account->comment)  g_strfreev (account->comment);
 	if (account->home)     g_free (account->home);
 	if (account->shell)    g_free (account->shell);
 	if (account->uid)      g_free (account->uid);
